const { Issuer, generators } = require('openid-client');
let client;
let nonce;
let state;
let code_verifier;

const {
    CLIENT_ID = 'SR_91uGHZXCp00zZcJ8A6',
    CLIENT_SECRET = '3ZfhJrhmYVMOw51UFzVdKU6F1JeIMm9P8GSx8FpWOdtUJ3KduSxOHVNgOPeez5_fcF-5l3e0lg3c7yJw8WfkIA',
    DISCOVER_URL = 'https://login.pre.nautilus.app/oidc/',
    END_SESSION_ENDPOINT = 'https://login.pre.nautilus.app/oidc/session/end'
} = process.env;

const express = require('express');
const app = express();
const port = 3000;

async function main() {
    // Descubre todos los endpoints del servidor OIDC automáticamente
    const issuer = await Issuer.discover(DISCOVER_URL);

    // Crea un cliente con las credenciales registradas en el servidor OIDC
    client = new issuer.Client({
        client_id: CLIENT_ID,
        client_secret: CLIENT_SECRET,
        redirect_uris: ['http://localhost:3000/cb'], // URL a la que el OP redirigirá
        response_types: ['code'],
    });

    nonce = generators.nonce();
    state = generators.state();
    code_verifier = generators.codeVerifier();
    const code_challenge = generators.codeChallenge(code_verifier);
    const authorizationUrl = client.authorizationUrl({
        scope: 'openid email profile',
        response_mode: 'query',
        nonce,
        state,
        code_verifier,
        code_challenge,
        code_challenge_method: 'S256',
    });

    console.log('Visit this URL in your browser:', authorizationUrl);
}

async function getUserInfo(tokenSet) {
    try {
        const userInfo = await client.userinfo(tokenSet.access_token);
        console.log('User Info:', userInfo);
    } catch (error) {
        console.error('Failed to fetch user info:', error);
    }
}

async function revokeAccessToken(tokenSet) {
    try {
        if (tokenSet.access_token) {
            await client.revoke(tokenSet.access_token);
            console.log('Access token revoked successfully');
        } else {
            console.log('No access token to revoke');
        }
    } catch (error) {
        console.error('Failed to revoke access token:', error);
    }
}

function logout(tokenSet, res) {
    const idTokenHint = tokenSet.id_token;
    const postLogoutRedirectUri = 'http://localhost:3000/post-logout';
    const logoutUrl = new URL(END_SESSION_ENDPOINT);
    logoutUrl.searchParams.append('id_token_hint', idTokenHint);
    logoutUrl.searchParams.append('post_logout_redirect_uri', postLogoutRedirectUri);

    res.redirect(logoutUrl.toString());
}

app.get('/cb', async (req, res) => {
    const params = client.callbackParams(req);
    const tokenSet = await client.callback('http://localhost:3000/cb', params, {
        nonce,
        state,
        code_verifier
    });
    console.log('Token Set:', tokenSet);

    // Llamada al userinfo_endpoint antes de la revocación
    console.log('Fetching user info before revocation...');
    await getUserInfo(tokenSet);

    // Revocar el token de acceso
    await revokeAccessToken(tokenSet);

    // Llamada al userinfo_endpoint después de la revocación
    console.log('Fetching user info after revocation...');
    await getUserInfo(tokenSet);

    // Llamar a la función de logout
    logout(tokenSet, res);
});

app.listen(port, () => {
    console.log(`Example app listening at http://localhost:${port}`);
});
app.get('/post-logout', (req, res) => {
    console.log('You have been logged out successfully.');
    res.send('You have been logged out successfully.');
});
main().catch(console.error);
